@@ -1,9 +1,14 @@
 /* SPDX-License-Identifier: GPL-2.0-only OR GPL-3.0-only */
-/* Copyright (c) 2022 Brett Sheffield <bacs@librecast.net> */
+/* Copyright (c) 2022-2023 Brett Sheffield <bacs@librecast.net> */
 
 #include <lcrq_pvt.h>
 #include <arpa/inet.h>
 #include <assert.h>
+#ifdef HAVE_ENDIAN_H
+#include <endian.h>
+#elif HAVE_SYS_ENDIAN_H
+#include <sys/endian.h>
+#endif
 #include <errno.h>
 #include <gf256.h>
 #include <stdlib.h>
@@ -237,19 +242,24 @@ fail:
 int rq_encode(rq_t *rq, void *data, size_t len)
 {
 	const size_t clen = (uint32_t)rq->L * rq->T;
-	size_t blklen, off;
 	const size_t maxblk = rq->src_part.IL * rq->T;
+	const size_t szC = clen * rq->Z;
+	size_t blklen, off;
 	uint8_t *base;
-	uint8_t padblk[maxblk];
+	uint8_t *padblk;
 	uint8_t *srcblk = data;
-	int rc = 0;
+	int rc = -1;
 
 	/* create storage for intermediate symbols (C) */
-	rq->C = malloc(clen * rq->Z);
-	memset(rq->C, 0, clen * rq->Z);
-	base = rq->C;
+	padblk = malloc(maxblk);
+	if (!padblk) goto exit_0;
+	base = malloc(szC);
+	if (!base) goto exit_1;
+	memset(base, 0, szC);
+	rq->C = base;
 	rq->obj = data;
 	rq->sym = data;
+	rc = 0;
 	for (uint8_t SBN = 0; SBN < rq->Z && !rc; SBN++) {
 		if (SBN < rq->src_part.JL) {
 			rq->K = rq->src_part.IL;
@@ -272,6 +282,9 @@ int rq_encode(rq_t *rq, void *data, size_t len)
 		srcblk += off;
 		len -= off;
 	}
+exit_1:
+	free(padblk);
+exit_0:
 	return rc;
 }
 
@@ -932,6 +945,42 @@ uint8_t rq_Al(const rq_t * const rq) { return rq->Al; }
 uint16_t rq_KP(const rq_t * const rq) { return rq->KP; }
 uint16_t rq_K(const rq_t * const rq) { return rq->K; }
 
+uint64_t rq_oti_F(rq_oti_t oti)
+{
+	return (be64toh(oti) >> 24);
+}
+
+uint16_t rq_oti_T(rq_oti_t oti)
+{
+	return be64toh(oti) & 0xffff;
+}
+
+uint8_t rq_oti_Z(rq_oti_t scheme)
+{
+	return be32toh(scheme) >> 24;
+}
+
+uint16_t rq_oti_N(rq_oti_t scheme)
+{
+	return (be32toh(scheme) & 0x00ffff00) >> 8;
+}
+
+uint8_t rq_oti_Al(rq_oti_t scheme)
+{
+	return be32toh(scheme) & 0xff;
+}
+
+int rq_oti(rq_t *rq, rq_oti_t *oti, rq_scheme_t *scheme)
+{
+	if (oti) {
+		*oti = htobe64((rq->F << 24) | rq->T);
+	}
+	if (scheme) {
+		*scheme = htobe32((rq->Z << 24) | (rq->N << 8) | rq->Al);
+	}
+	return 0;
+}
+
 void rq_free(rq_t *rq)
 {
 	if (rq) {
@@ -963,8 +1012,7 @@ void rq_block(rq_t *rq)
 	rq->B = rq->W - rq->S;
 
 	/* P1 denotes the smallest prime number greater than or equal to P */
-	rq->P1 = rq->P;
-	while (!isprime(rq->P1)) rq->P1++;
+	for (rq->P1 = rq->P; !isprime(rq->P1); rq->P1++);
 
 	/* verify primes */
 	assert(isprime(rq->P1));
